/* ===========================================================
 * nl.js
 * Dutch translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Danny Hiemstra
 *          Github : https://github.com/dhiemstra
 */

jQuery.trumbowyg.langs.nl = {
    viewHTML: 'HTML bekijken',

    undo: 'Ongedaan maken',
    redo: 'Opnieuw',

    formatting: 'Opmaak',
    p: 'Paragraaf',
    blockquote: 'Citaat',
    code: 'Code',
    header: 'Kop',

    bold: 'Vet',
    italic: 'Cursief',
    strikethrough: 'Doorhalen',
    underline: 'Onderlijnen',

    strong: 'Sterk',
    em: 'Nadruk',
    del: 'Verwijderd',

    unorderedList: 'Ongenummerde lijst',
    orderedList: 'Genummerde lijst',

    insertImage: 'Afbeelding invoegen',
    insertVideo: 'Video invoegen',
    link: 'Link',
    createLink: 'Link maken',
    unlink: 'Link verwijderen',

    justifyLeft: 'Links uitlijnen',
    justifyCenter: 'Centreren',
    justifyRight: 'Rechts uitlijnen',
    justifyFull: 'Uitvullen',

    horizontalRule: 'Horizontale lijn',
    removeFormat: 'Opmaak verwijderen',

    fullscreen: 'Volledig scherm',

    close: 'Sluiten',

    submit: 'Opslaan',
    reset: 'Annuleren',

    required: 'Verplicht',
    description: 'Omschrijving',
    title: 'Titel',
    text: 'Tekst',
    target: 'Doel',
    width: 'Breedte'
};
