/**
 * Trumbowyg v2.18.0 - A lightweight WYSIWYG editor
 * Trumbowyg plugin stylesheet
 * ------------------------
 * @link http://alex-d.github.io/Trumbowyg
 * @license MIT
 * @author Alexandre Demode (Alex-D)
 *         Twitter : @AlexandreDemode
 *         Website : alex-d.fr
 */

.trumbowyg-dropdown-emoji {
  width: 265px;
  padding: 7px 0 7px 5px; }

.trumbowyg-dropdown-emoji svg {
  display: none !important; }

.trumbowyg-dropdown-emoji button {
  display: block;
  position: relative;
  float: left;
  height: 26px;
  width: 26px;
  padding: 0;
  margin: 2px;
  line-height: 24px;
  text-align: center; }
  .trumbowyg-dropdown-emoji button:hover::after, .trumbowyg-dropdown-emoji button:focus::after {
    display: block;
    position: absolute;
    top: -5px;
    left: -5px;
    height: 27px;
    width: 27px;
    background: inherit;
    box-shadow: #000 0 0 2px;
    z-index: 10;
    background-color: transparent; }

.trumbowyg .emoji {
  width: 22px;
  height: 22px;
  display: inline-block; }
