/**
 * Trumbowyg v2.18.0 - A lightweight WYSIWYG editor
 * Default stylesheet for Trumbowyg editor plugin
 * ------------------------
 * @link http://alex-d.github.io/Trumbowyg
 * @license MIT
 * @author Alexandre Demode (Alex-D)
 *         Twitter : @AlexandreDemode
 *         Website : alex-d.fr
 */

.mathMlContainer {
	
	position: relative;
	
	&[inline="false"] {
		display: block;
		width: 100%;
	}
	
	&:hover {
		pointer-events: none;
		&:after {
			content: '\270E';
			pointer-events: auto;
			position: absolute;
			left: 0px;
			top: 0px;
			background-color: #ffffffd4;
			cursor: pointer;
			bottom: 0px;
			right: 0px;
			box-shadow: 0 0 5px 5px #ffffffd4;
		}
	}
}
