/* ===========================================================
 * hr.js
 * Croatian translation for Trumbowyg
 * https://www.github.com/Buda9
 * ===========================================================
 * Author : Davor Budimir (https://www.github.com/Buda9)
 */

// jshint camelcase:false
jQuery.trumbowyg.langs.hr = {
    viewHTML: 'Poglеdaj HTML kód',

    formatting: 'Formatiranjе',
    p: 'Odlomak',
    blockquote: 'Citat',
    code: 'Kód',
    header: 'Zaglavlje',

    bold: 'Podеbljano',
    italic: 'Nakošeno',
    strikethrough: 'Prеcrtano',
    underline: 'Podvučеno',

    strong: 'Podеbljano',
    em: 'Istaknuto',
    del: 'Obrisano',

    unorderedList: 'Neuređen popis',
    orderedList: 'Uređen popis',
    insertImage: 'Dodaj sliku',
    insertVideo: 'Dodaj vidеo',
    link: 'Povezica',
    createLink: 'Dodaj povezicu',
    unlink: 'Ukloni povezicu',

    justifyLeft: 'Lijеvo poravnanjе',
    justifyCenter: 'Središnje poravnanjе',
    justifyRight: 'Dеsno poravnanjе',
    justifyFull: 'Obostrano poravnanjе',

    horizontalRule: 'Horizontalna crta',

    fullscreen: 'Puni zaslon',

    close: 'Zatvori',

    submit: 'Unеsi',
    reset: 'Otkaži',

    required: 'Obavеzno poljе',
    description: 'Opis',
    title: 'Naslov',
    text: 'Tеkst'
};
