/* ===========================================================
 * ca.js
 * Catalan translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Àlfons Sánchez (alsanan)
 *          Twitter : @alsanan
 *          Website : about.me/alsanan
 *          Github : https://github.com/alsanan
 */

jQuery.trumbowyg.langs.ca = {
    viewHTML: 'Veure HTML',

    formatting: 'Formatar',
    p: 'Paragraf',
    blockquote: 'Citació',
    code: 'Codi',
    header: 'Títol',

    bold: 'Negreta',
    italic: 'Itàlica',
    strikethrough: 'Suprimir',
    underline: 'Subratllat',

    strong: 'Forta',
    em: 'Èmfasi',
    del: 'Apagar',

    unorderedList: 'Lista desordenada',
    orderedList: 'Lista ordenada',

    insertImage: 'Inserir imatge',
    insertVideo: 'Inserir vídeo',
    link: 'Enllaç',
    createLink: 'Crear un enllaç',
    unlink: 'Eliminar enllaç',

    justifyLeft: 'Alinear a esquerra',
    justifyCenter: 'Centrar',
    justifyRight: 'Alinear a dreta',
    justifyFull: 'Justificar',

    horizontalRule: 'Inserir separador horitzontal',

    fullscreen: 'Pantalla completa',

    close: 'Tancar',

    submit: 'Enviar',
    reset: 'Reiniciar',

    required: 'Obligatori',
    description: 'Descripció',
    title: 'Títol',
    text: 'Text'
};
