/* ===========================================================
 * ko.js
 * Korean translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : SeungWoo Chae (SDuck4)
 *          Github : https://github.com/SDuck4
 */

jQuery.trumbowyg.langs.ko = {
    viewHTML: 'HTML 보기',

    undo: '되돌리기',
    redo: '다시 실행',

    formatting: '스타일',
    p: '본문',
    blockquote: '인용문',
    code: '코드',
    header: '제목',

    bold: '진하게',
    italic: '기울임',
    strikethrough: '취소선',
    underline: '밑줄',

    strong: '중요',
    em: '강조',
    del: '삭제',

    superscript: '위 첨자',
    subscript: '아래 첨자',

    unorderedList: '기호 목록',
    orderedList: '번호 목록',

    insertImage: '그림 넣기',
    link: '링크',
    createLink: '링크 넣기',
    unlink: '링크 지우기',

    justifyLeft: '왼쪽 정렬',
    justifyCenter: '가운데 정렬',
    justifyRight: '오른쪽 정렬',
    justifyFull: '양쪽 정렬',

    horizontalRule: '구분선 넣기',
    removeformat: '글꼴 효과 지우기',

    fullscreen: '전체 화면',

    close: '닫기',

    submit: '확인',
    reset: '취소',

    required: '필수 입력',
    description: '설명',
    title: '툴팁',
    text: '내용',
    target: '타겟',
    width: '너비'
};
