/* ===========================================================
 * ph.js
 * Filipino translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : @leogono
 */

jQuery.trumbowyg.langs.ph = {
    viewHTML: 'Tumingin sa HTML',

    formatting: 'Formatting',
    p: 'Talata',
    blockquote: 'Blockquote',
    code: 'Kowd',
    header: 'Header',

    bold: 'Makapal',
    italic: 'Hilig',
    strikethrough: 'Strikethrough',
    underline: 'Salungguhit',

    strong: 'Malakas',
    em: 'Hilig',
    del: 'Tinanggal',

    unorderedList: 'Hindi nakahanay na listahan',
    orderedList: 'Nakahanay na listahan',

    insertImage: 'Ilagay ang larawan',
    insertVideo: 'Ilagay ang video',
    link: 'Koneksyon',
    createLink: 'Iugnay',
    unlink: 'Tanggalin ang koneksyon',

    justifyLeft: 'Ihanay sa kaliwa',
    justifyCenter: 'Ihanay sa gitna',
    justifyRight: 'Ihanay sa kanan',
    justifyFull: 'Ihanay sa kaliwa at kanan',

    horizontalRule: 'Pahalang na linya',

    fullscreen: 'Fullscreen',

    close: 'Isara',

    submit: 'Ipasa',
    reset: 'I-reset',

    required: 'Kailangan',
    description: 'Paglalarawan',
    title: 'Pamagat',
    text: 'Teksto'
};
