/**
 * Trumbowyg v2.18.0 - A lightweight WYSIWYG editor
 * Trumbowyg plugin stylesheet
 * ------------------------
 * @link http://alex-d.github.io/Trumbowyg
 * @license MIT
 * @author Alexandre Demode (Alex-D)
 *         Twitter : @AlexandreDemode
 *         Website : alex-d.fr
 */

.trumbowyg-editor table {
  width: 100%; }
  .trumbowyg-editor table td {
    border: 1px dotted #e7eaec;
    padding: 8px; }

.trumbowyg-dropdown-table table {
  margin: 10px;
  display: inline-block; }

.trumbowyg-dropdown-table table td {
  display: inline-block;
  height: 20px;
  width: 20px;
  margin: 1px;
  padding: 0;
  background-color: #fff;
  box-shadow: 0 0 0 1px #cecece inset; }
  .trumbowyg-dropdown-table table td.active {
    background-color: #00b393;
    box-shadow: none;
    cursor: pointer; }

.trumbowyg-dropdown-table .trumbowyg-table-size {
  text-align: center; }
