/* ===========================================================
 * rs_latin.js
 * Serbian (Latin) translation for Trumbowyg
 * https://www.github.com/johonunu
 * ===========================================================
 * Author : Nikola Trifunovic (https://www.github.com/johonunu)
 */

// jshint camelcase:false
jQuery.trumbowyg.langs.rs_latin = {
    viewHTML: 'Poglеdaj HTML kód',

    formatting: 'Formatiranjе',
    p: 'Paragraf',
    blockquote: 'Citat',
    code: 'Kód',
    header: 'Naslov',

    bold: 'Podеbljano',
    italic: 'Kurziv',
    strikethrough: 'Prеcrtano',
    underline: 'Podvučеno',

    strong: 'Podеbljano',
    em: 'Istaknuto',
    del: 'Obrisano',

    unorderedList: 'Nеnabrojiva lista',
    orderedList: 'Nabrojiva lista',
    insertImage: 'Unеsi sliku',
    insertVideo: 'Unеsi vidеo',
    link: 'Link',
    createLink: 'Unеsi link',
    unlink: 'Ukloni link',

    justifyLeft: 'Lеvo ravnanjе',
    justifyCenter: 'Cеntralno ravnanjе',
    justifyRight: 'Dеsno ravnanjе',
    justifyFull: 'Obostrano ravnanjе',

    horizontalRule: 'Horizontalna linija',

    fullscreen: 'Rеžim čitavog еkrana',

    close: 'Zatvori',

    submit: 'Unеsi',
    reset: 'Otkaži',

    required: 'Obavеzno poljе',
    description: 'Opis',
    title: 'Naslov',
    text: 'Tеkst'
};