/* ===========================================================
 * bg.js
 * Bulgarian translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Aleksandar Dimitrov
 */

jQuery.trumbowyg.langs.bg = {
    viewHTML: 'Прегледай HTML',

    formatting: 'Форматиране',
    p: 'Параграф',
    blockquote: 'Цитат',
    code: 'Код',
    header: 'Заглавие',

    bold: 'Удебелен',
    italic: 'Наклонен',
    strikethrough: 'Зачеркнат',
    underline: 'Подчертан',

    strong: 'Удебелен',
    em: 'Наклонен',
    del: 'Зачеркнат',

    unorderedList: 'Обикновен списък',
    orderedList: 'Номериран списък',

    insertImage: 'Добави изображение',
    insertVideo: 'Добави видео',
    link: 'Връзка',
    createLink: 'Създай връзка',
    unlink: 'Премахни връзката',

    justifyLeft: 'Подравни от ляво',
    justifyCenter: 'Центрирай',
    justifyRight: 'Подравни от дясно',
    justifyFull: 'Подравни по ширина',

    horizontalRule: 'Хоризонтална линия',

    fullscreen: 'На цял екран',

    close: 'Затвори',

    submit: 'Впиши',
    reset: 'Отмени',

    required: 'Задължително',
    description: 'Описание',
    title: 'Заглавие',
    text: 'Текст'
};
