jQuery.trumbowyg.langs.sl = {
    viewHTML: 'Prikaži HTML',

    formatting: 'Formatiranje',
    p: 'Odstavek',
    blockquote: 'Citat',
    code: 'Koda',
    header: 'Glava',

    bold: 'Krepko',
    italic: 'Ležeče',
    strikethrough: 'Prečrtano',
    underline: 'Podčrtano',

    strong: 'Odebljeno',
    em: 'Poudarjeno',
    del: 'Izbrisano',

    unorderedList: 'Neoštevilčen seznam',
    orderedList: 'Oštevilčen seznam',

    image: 'Slika',
    insertImage: 'Vzstavi sliko',
    insertVideo: 'Vzstavi video',
    link: 'Povezava',
    createLink: 'Vzstavi povezavo',
    unlink: 'Odstrani povezavo',

    justifyLeft: 'Poravnava levo',
    justifyCenter: 'Poravnaj na sredino',
    justifyRight: 'Poravnava desno',
    justifyFull: 'Obojestranska poravnava',

    horizontalRule: 'Vstavite vodoravno črto',
    removeformat: 'Odstrani format',
    
    fullscreen: 'Celozaslonski način',

    close: 'Zapri',

    submit: 'Potrdi',
    reset: 'Resetiraj',

    required: 'Erforderlich',
    description: 'Opis',
    title: 'Naslov',
    text: 'Besedilo'
};
