/* ===========================================================
 * he.js
 * Hebrew translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Udi Doron (udidoron)
 *          Github : https://github.com/udidoron
 */

jQuery.trumbowyg.langs.he = {
    _dir: 'rtl',

    viewHTML: 'צפה ב-HTML',

    formatting: 'פורמט',
    p: 'פסקה',
    blockquote: 'ציטוט',
    code: 'קוד',
    header: 'ראשית',

    bold: 'מודגש',
    italic: 'נטוי',
    strikethrough: 'קו חוצה',
    underline: 'קו תחתון',

    strong: 'בולט',
    em: 'הדגשה',
    del: 'נמחק',

    unorderedList: 'רשימה ללא סדר',
    orderedList: 'רשימה מסודרת',

    insertImage: 'הכנס תמונה',
    insertVideo: 'הכנס סרטון',
    link: 'קישור',
    createLink: 'צור קישור',
    unlink: 'הסר קישור',

    justifyLeft: 'ישר לשמאל',
    justifyCenter: 'מרכז',
    justifyRight: 'ישר לימין',
    justifyFull: 'ישר לשני הצדדים',

    horizontalRule: 'הכנס קו אופקי',

    fullscreen: 'מסך מלא',

    close: 'סגור',

    submit: 'שלח',
    reset: 'אתחל מחדש',

    required: 'נחוץ',
    description: 'תיאור',
    title: 'כותרת',
    text: 'טקסט'
};