/* ===========================================================
 * fa.js
 * Persian translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Kiarash Soleimanzadeh
 *          Github : https://github.com/kiyarash
 *          Email : kiarash.s@hotmail.com
 */

jQuery.trumbowyg.langs.fa = {
    _dir: 'rtl',
    
    viewHTML: 'نمایش کد اچ تی ام ال',

    formatting: 'قالب بندی',
    p: 'پاراگراف',
    blockquote: 'نقل قول',
    code: 'کد',
    header: 'سر تیتر',

    bold: 'ضخیم',
    italic: 'مورب',
    strikethrough: 'میان خط دار',
    underline: 'زیر خط دار',

    strong: 'برجسته',
    em: 'مورب',
    del: 'حذف شده',

    unorderedList: 'لیست نامرتب',
    orderedList: 'لیست مرتب',

    insertImage: 'درج تصویر',
    insertVideo: 'درج ویدئو',
    link: 'لینک',
    createLink: 'درج لینک',
    unlink: 'حذف لینک',

    justifyLeft: 'تراز به چپ',
    justifyCenter: 'تراز به وسط',
    justifyRight: 'تراز به راست',
    justifyFull: 'تراز به چپ و راست',

    horizontalRule: 'درج خط افقی',

    fullscreen: 'تمام صفحه',

    close: 'بستن',

    submit: 'تائید',
    reset: 'انصراف',

    required: 'اجباری',
    description: 'توضیحات',
    title: 'عنوان',
    text: 'متن'
};
