/* ===========================================================
 * ar.js
 * Arabic translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Abo Mokh ahmed (abomokhahmed)
 *          Github : https://github.com/abomokhahmed
 * Reviewed by : Abdellah Chadidi (chadidi)
 *          Github : https://github.com/chadidi
 */

jQuery.trumbowyg.langs.ar = {
    _dir: 'rtl',

    viewHTML: 'إعرض-HTML',

    undo: 'تراجع',
    redo: 'إعادة',

    formatting: 'تنسيق',

    p: 'فقرة',
    blockquote: 'اقتباس',
    code: 'كود',
    header: 'رأس',

    bold: 'عريض',
    italic: 'مائل',
    strikethrough: 'مشطوب',
    underline: 'خطّ سفلي',

    strong: 'بارز',
    em: 'تغميق',
    del: 'حذف',

    superscript: 'الأس',
    subscript: 'أس سفلي',

    unorderedList: 'قائمة غير مرتّبة',
    orderedList: 'قائمة مرتّبة',

    insertImage: 'إدراج صورة',
    insertVideo: 'إدراج فيديو',
    link: 'رابط',
    createLink: 'انشاء رابط',
    unlink: 'حذف رابط',

    justifyLeft: 'تصحيح للشمال',
    justifyCenter: 'توسيط',
    justifyRight: 'تصحيح لليمين',
    justifyFull: 'تصحيح لكلا الإتّجاهين',

    horizontalRule: 'إدراج خطّ أفقي',

    fullscreen: 'ملء الشاشة',

    close: 'إغلاق',

    submit: 'إرسال',
    reset: 'إعادة تعيين',

    required: 'إلزامي',
    description: 'وصف',
    title: 'عنوان',
    text: 'نصّ',
    target: 'الهدف'
};
