/* ===========================================================
 * my.js
 * Malaysian translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : JohnPozy
 */

jQuery.trumbowyg.langs.id = {
    viewHTML: 'Lihat HTML',

    formatting: 'Pemformatan',
    p: 'Perenggan',
    blockquote: 'Blockquote',
    code: 'Kod',
    header: 'Pengepala',

    bold: 'Tebal',
    italic: 'Condong',
    strikethrough: 'Garis batal',
    underline: 'Garis bawah',

    strong: 'Kuat',
    em: 'Condong',
    del: 'Hapus',

    unorderedList: 'Senarai tidak tertib',
    orderedList: 'Senarai tertib',

    insertImage: 'Masukkan imej',
    insertVideo: 'Masukkan video',
    link: 'Pautan',
    createLink: 'Cipta pautan',
    unlink: 'Hapus pautan',

    justifyLeft: 'Mengimbangkan ke kiri',
    justifyCenter: 'Mengimbangkan ke tengah',
    justifyRight: 'Mengimbangkan ke kanan',
    justifyFull: 'Mengimbangkan ke kiri dan kanan',

    horizontalRule: 'Masukkan garis mendatar',

    fullscreen: 'Skrin penuh',

    close: 'Tutup',

    submit: 'Hantar',
    reset: 'Batal',

    required: 'Diperlukan',
    description: 'Perihal',
    title: 'Tajuk',
    text: 'Teks'
};
