/* ===========================================================
 * da.js
 * Danish translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Christian Pedersen
 *          Github : https://github.com/chripede
 */

jQuery.trumbowyg.langs.da = {
    viewHTML: 'Vis HTML',

    undo: 'Fortryd',
    redo: 'Anuller fortryd',

    formatting: 'Formattering',
    p: 'Afsnit',
    blockquote: 'Citat',
    code: 'Kode',
    header: 'Overskrift',

    bold: 'Fed',
    italic: 'Kursiv',
    strikethrough: 'Gennemstreg',
    underline: 'Understreg',

    strong: 'Vigtig',
    em: 'Fremhæv',
    del: 'Slettet',

    superscript: 'Hævet skrift',
    subscript: 'Sænket skrift',

    unorderedList: 'Uordnet liste',
    orderedList: 'Ordnet liste',

    insertImage: 'Indsæt billede',
    insertVideo: 'Indsæt video',
    link: 'Link',
    createLink: 'Indsæt link',
    unlink: 'Fjern link',

    justifyLeft: 'Venstrestil',
    justifyCenter: 'Centrer',
    justifyRight: 'Højrestil',
    justifyFull: 'Lige margener',

    horizontalRule: 'Horisontal linie',
    removeformat: 'Ryd formattering',

    fullscreen: 'Fuld skærm',

    close: 'Luk',

    submit: 'Bekræft',
    reset: 'Annuller',

    required: 'Påkrævet',
    description: 'Beskrivelse',
    title: 'Titel',
    text: 'Tekst',
    target: 'Mål',
    width: 'Bredde'
};