/* ===========================================================
 * ru.js
 * Russion translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Yuri Lya
 */

jQuery.trumbowyg.langs.ru = {
    viewHTML: 'Посмотреть HTML',

    undo: 'Отменить',
    redo: 'Повторить',

    formatting: 'Форматирование',
    p: 'Обычный',
    blockquote: 'Цитата',
    code: 'Код',
    header: 'Заголовок',

    bold: 'Полужирный',
    italic: 'Курсив',
    strikethrough: 'Зачеркнутый',
    underline: 'Подчеркнутый',

    strong: 'Полужирный',
    em: 'Курсив',
    del: 'Зачеркнутый',

    superscript: 'Надстрочный',
    subscript: 'Подстрочный',

    unorderedList: 'Обычный список',
    orderedList: 'Нумерованный список',

    insertImage: 'Вставить изображение',
    insertVideo: 'Вставить видео',
    link: 'Ссылка',
    createLink: 'Вставить ссылку',
    unlink: 'Удалить ссылку',

    justifyLeft: 'По левому краю',
    justifyCenter: 'По центру',
    justifyRight: 'По правому краю',
    justifyFull: 'По ширине',

    horizontalRule: 'Горизонтальная линия',
    removeformat: 'Очистить форматирование',

    fullscreen: 'Во весь экран',

    close: 'Закрыть',

    submit: 'Вставить',
    reset: 'Отменить',

    required: 'Обязательное',
    description: 'Описание',
    title: 'Подсказка',
    text: 'Текст'
};
