/* ===========================================================
 * sv.js
 * Swedish translation for Trumbowyg
 * http://www.tim-international.net
 * ===========================================================
 * Author : T. Almroth
 * Github : https://github.com/timint
 *
 * Review : M Hagberg
 * Github : https://github.com/pestbarn
 */

jQuery.trumbowyg.langs.sv = {
    viewHTML: 'Visa HTML',

    formatting: 'Formatering',
    p: 'Paragraf',
    blockquote: 'Citat',
    code: 'Kod',
    header: 'Rubrik',

    bold: 'Fet',
    italic: 'Kursiv',
    strikethrough: 'Genomstruken',
    underline: 'Understruken',

    strong: 'Fet',
    em: 'Kursiv',
    del: 'Rensa formatering',

    unorderedList: 'Punktlista',
    orderedList: 'Numrerad lista',

    insertImage: 'Infoga bild',
    insertVideo: 'Infoga video',
    link: 'Länk',
    createLink: 'Infoga länk',
    unlink: 'Ta bort länk',

    justifyLeft: 'Vänsterjustera',
    justifyCenter: 'Centrera',
    justifyRight: 'Högerjustera',
    justifyFull: 'Marginaljustera',

    horizontalRule: 'Horisontell linje',

    fullscreen: 'Fullskärm',

    close: 'Stäng',

    submit: 'Bekräfta',
    reset: 'Återställ',

    required: 'Obligatorisk',
    description: 'Beskrivning',
    title: 'Titel',
    text: 'Text'
};
