/* ===========================================================
 * el.js
 * Greek translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Merianos Nikos
 *          Twitter : @_webresources
 *          Website : http://www.wp-lion.com
 *          LinkedIn: https://gr.linkedin.com/in/merianosnikos
 *          Behance: https://www.behance.net/web_design_blog
 */

jQuery.trumbowyg.langs.el = {
    viewHTML: 'Προβολή κώδικα HTML',

    formatting: 'Μορφοποίηση',
    p: 'Παράγραφος',
    blockquote: 'Παράθεση',
    code: 'Κώδικας',
    header: 'Επικεφαλίδα',

    bold: 'Έντονα',
    italic: 'Πλάγια',
    strikethrough: 'Διαγραφή',
    underline: 'Υπογράμμιση',

    strong: 'Έντονα',
    em: 'Πλάγια',
    del: 'Διαγραφή',

    unorderedList: 'Αταξινόμητη λίστα',
    orderedList: 'Ταξινομημένη λίστα',

    insertImage: 'Εισαγωγή εικόνας',
    insertVideo: 'Εισαγωγή βίντεο',
    link: 'Σύνδεσμος',
    createLink: 'Δημιουργία συνδέσμου',
    unlink: 'Διαγραφή συνδέσμου',

    justifyLeft: 'Στοίχιση αριστερά',
    justifyCenter: 'Στοίχιση στο κέντρο',
    justifyRight: 'Στοίχιση δεξιά',
    justifyFull: 'Πλήρης στοίχιση',

    horizontalRule: 'Οριζόντια γραμμή',
    removeformat: 'Καθαρισμός μορφοποίησης',

    fullscreen: 'Πλήρης οθόνη',

    close: 'Κλείσιμο',

    submit: 'Υποβολή',
    reset: 'Επαναφορά',

    required: 'Απαραίτητο',
    description: 'Περιγραφή',
    title: 'Τίτλος',
    text: 'Κείμενο'
};
