/* ===========================================================
 * lt.js
 * Lithuanian translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Justas Brazauskas
 */

jQuery.trumbowyg.langs.lt = {
    viewHTML: 'Žiūrėti HTML',

    formatting: 'Formatuoti',
    p: 'Paragrafas',
    blockquote: 'Citata',
    code: 'Kodas',
    header: 'Antraštė',

    bold: 'Paryškinti',
    italic: 'Kursyvuoti',
    strikethrough: 'Perbraukti',
    underline: 'Pabrėžti',

    strong: 'Paryškinti',
    em: 'Pabrėžti',
    del: 'Trinti',

    unorderedList: 'Sąrašas',
    orderedList: 'Numeruotas sąrašas',

    insertImage: 'Pridėti vaizdą',
    insertVideo: 'Pridėti video',
    link: 'Nuoroda',
    createLink: 'Kurti nuorodą',
    unlink: 'Ištrinti nuorodą',

    justifyLeft: 'Lyginti kairėn',
    justifyCenter: 'Lygiuoti Centre',
    justifyRight: 'Lyginti dešinėn',
    justifyFull: 'Centruoti',

    horizontalRule: 'Horizontali linija',

    fullscreen: 'Pilnas ekranas',

    close: 'Uždaryti',

    submit: 'Siųsti',
    reset: 'Atšaukti',

    required: 'Privaloma',
    description: 'Aprašymas',
    title: 'Pavadinimas',
    text: 'Tekstas',

    removeformat: 'Pašalinti formatavimą',

    superscript: 'Viršutinis indeksas',
    subscript: 'Apatinis indeksas',
};
