/* ===========================================================
 * hu.js
 * Hungarian translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Peter MATO
 *          Web: http://fixme.hu
 *          GitHub: https://github.com/matopeter
 */

jQuery.trumbowyg.langs.hu = {
    viewHTML: 'HTML nézet',

    formatting: 'Stílusok',

    p: 'Bekezdés',
    blockquote: 'Idézet',
    code: 'Kód',
    header: 'Címsor',

    bold: 'Félkövér',
    italic: 'Dőlt',
    strikethrough: 'Áthúzott',
    underline: 'Aláhúzott',

    strong: 'Vastag',
    em: 'Kiemelt',
    del: 'Törölt',

    unorderedList: 'Felsorolás',
    orderedList: 'Számozás',

    insertImage: 'Kép beszúrása',
    insertVideo: 'Video beszúrása',
    link: 'Link',
    createLink: 'Link létrehozása',
    unlink: 'Link eltávolítása',

    justifyLeft: 'Balra igazítás',
    justifyCenter: 'Középre igazítás',
    justifyRight: 'Jobbra igazítás',
    justifyFull: 'Sorkizárt',

    horizontalRule: 'Vízszintes vonal',

    fullscreen: 'Teljes képernyő',
    close: 'Bezár',

    submit: 'Beküldés',
    reset: 'Alaphelyzet',

    required: 'Kötelező',
    description: 'Leírás',
    title: 'Cím',
    text: 'Szöveg',

    removeformat: 'Formázás eltávolítása'
};
