/* ===========================================================
 * es.js
 * Spanish translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Moisés Márquez
 *          Email : moises.marquez.g@gmail.com
 */

jQuery.trumbowyg.langs.es = {
    viewHTML: 'Ver HTML',

    undo: 'Deshacer',
    redo: 'Rehacer',

    formatting: 'Formato',
    p: 'Párrafo',
    blockquote: 'Cita',
    code: 'Código',
    header: 'Título',

    bold: 'Negrita',
    italic: 'Cursiva',
    strikethrough: 'Tachado',
    underline: 'Subrayado',

    strong: 'Negrita',
    em: 'Énfasis',
    del: 'Borrar',

    superscript: 'Sobrescrito',
    subscript: 'Subíndice',

    unorderedList: 'Lista Desordenada',
    orderedList: 'Lista Ordenada',

    insertImage: 'Insertar una imagen',
    insertVideo: 'Insertar un vídeo',
    link: 'Enlace',
    createLink: 'Insertar un enlace',
    unlink: 'Suprimir un enlace',

    justifyLeft: 'Izquierda',
    justifyCenter: 'Centrar',
    justifyRight: 'Derecha',
    justifyFull: 'Justificado',

    horizontalRule: 'Insertar separador horizontal',
    removeformat: 'Eliminar formato',

    fullscreen: 'Pantalla completa',

    close: 'Cerrar',

    submit: 'Enviar',
    reset: 'Cancelar',

    required: 'Obligatorio',
    description: 'Descripción',
    title: 'Título',
    text: 'Texto',
    target: 'Target'
};
