/* ===========================================================
 * tr.js
 * Turkish translation for Trumbowyg
 * http://alex-d.github.com/Trumbowyg
 * ===========================================================
 * Author : Emrah Bilbay (munzur)
 *          Github : https://github.com/munzur
 */

jQuery.trumbowyg.langs.tr = {
    viewHTML: 'HTML Kodu',

    formatting: 'Biçimlendirme',
    p: 'Paragraf',
    blockquote: 'Alıntı',
    code: 'Kod',
    header: 'Başlık',

    bold: 'Kalın',
    italic: 'İtalik',
    strikethrough: 'Üzeri çizgili',
    underline: 'Altı çizgili',

    strong: 'Koyu',
    em: 'Vurgulu',
    del: 'Üzeri çizgili',

    unorderedList: 'Simgeli liste',
    orderedList: 'Numaralı liste',

    insertImage: 'Resim yerleştir',
    insertVideo: 'Video yerleştir',
    link: 'Link',
    createLink: 'Link yerleştir',
    unlink: 'Linki sil',

    justifyLeft: 'Sola hizala',
    justifyCenter: 'Ortaya hizala',
    justifyRight: 'Sağa hizala',
    justifyFull: 'Yasla',

    horizontalRule: 'Yatay satır ekle',

    fullscreen: 'Tam ekran',

    close: 'Kapat',

    submit: 'Onayla',
    reset: 'Sıfırla',

    required: 'Gerekli',
    description: 'Açıklama',
    title: 'Başlık',
    text: 'Metin'
};
